import java.awt.*;
import java.awt.event.*;

//main class
//create graphics window, initialize event handlers, start animation process
public class BSP {
	public static void main(String[] args) {
		//create graphics window
		Figure f = new Figure("BSP Graphics Window");
		//add listeners for mouse events
		f.addWindowListener(new MyWindowAdapter());
		f.addMouseListener(new MyMouseAdapter());
		f.addMouseMotionListener(new MyMouseMotionAdapter());
		f.addKeyListener(new MyKeyAdapter());
		//start separate thread for animation
		new Animation(f).start();
	}
}

//handle window closing event
class MyWindowAdapter extends WindowAdapter {
	public void windowClosing(WindowEvent event) {
		System.exit(0);
	}
}

//handle mouse click events
class MyMouseAdapter extends MouseAdapter {
	public void mouseClicked(MouseEvent e) {
		Figure f = (Figure)e.getComponent();
		f.fill = !f.fill; //toggle fill switch
	}
}

//handle mouse motion events
class MyMouseMotionAdapter extends MouseMotionAdapter {
	public void mouseMoved(MouseEvent e) {
		Figure f = (Figure)e.getComponent();
		//mouse position determines velocity of rotation
		Point p = e.getPoint();
		f.velocity = new TwoDPoint(p.x-250,p.y-250);
	}
}

//handle key events
class MyKeyAdapter extends KeyAdapter {
	public void keyPressed(KeyEvent e) {
		Figure f = (Figure)e.getComponent();
		switch (e.getKeyCode()) {
		case KeyEvent.VK_UP: //uparrow key--move forward
			f.distanceVelocity = f.advance;
			break;
		case KeyEvent.VK_DOWN: //downarrow key--move backward
			f.distanceVelocity = -f.advance;
			break;
		}
	}
	//key released--stop
	public void keyReleased(KeyEvent e) {
		Figure f = (Figure)e.getComponent();
		f.distanceVelocity = 0;
	}
}

//animation thread
//this thread will continuously render the image
class Animation extends Thread {
	Figure f;
	Animation(Figure f) {
		this.f = f;
	}		
	public void run() {
		f.animate(this);
	}
}

