import java.awt.*;
import java.awt.image.*;
import java.util.*;

class Figure extends Frame {
	Cubes cubes = new Cubes(); //the data for the image
	boolean fill = false; //solid or skeleton?--toggle by mouse click
	static final ThreeDPoint baseLightSource = new ThreeDPoint(10,20,30);
	double viewerDistance = 10; //initial viewer distance
	ThreeDPoint baseViewerPosition = new ThreeDPoint(viewerDistance,0,0); //initial position of viewer
//	static final double minViewerDistance = Math.sqrt(27); //close as I'll let you come
	static final double minViewerDistance = Double.NEGATIVE_INFINITY; //close as I'll let you come
	double distanceVelocity = 0; //approach speed
	static final double advance = -.1; //constant approach speed
	Matrix orientation = new Matrix(); //orientation matrix--a 3D orthonormal matrix
	Matrix gamma = new Matrix(); //auxiliary matrix used in calculations
	Matrix delta = new Matrix(); //"
	TwoDPoint velocity = new TwoDPoint(0,0); //rotation vector determined by mouse position
	Image image; //off-screen image buffer
	Graphics buffer; //graphics of off-screen buffer
	Graphics screen; //on-screen graphics

	Figure(String title) {
		super(title);
		setSize(new Dimension(500,500));
		setBackground(Color.black);
		show();
		//initialize screen graphics and image buffer
		screen = getGraphics();
		screen.translate(250,250);
		image = createImage(500,500);
		buffer = image.getGraphics();
		buffer.translate(250,250);
	}
	
	//calculate current position of viewer relative to image
	ThreeDPoint viewerPosition() {
		return orientation.mult(baseViewerPosition);
	}
	
	//calculate current position of light source relative to image
	ThreeDPoint lightSource() {
		return orientation.mult(baseLightSource);
	}
	
	//render one square
	//called from BSP tree render routine
	void render(Square s) {
		ThreeDPoint p = viewerPosition();
		//compute brightness factor and scale color by it
		float intensity = fill?(float)(lightSource().cos(s.normal)+1)/2:1;
		float r = intensity*s.color.getRed()/256;
		float g = intensity*s.color.getGreen()/256;
		float b = intensity*s.color.getBlue()/256;
		buffer.setColor(new Color(r,g,b));
		if (fill) { //if drawing solid--toggle by mouse click
			buffer.fillPolygon(s.project(orientation,baseViewerPosition));
		} else { //if drawing skeleton
			buffer.drawPolygon(s.project(orientation,baseViewerPosition));
		}
	}

	//continuously render the image, updating orientation between renderings
	void animate(Thread thread) {
		while (true) {
			//the updated orientation is determined by the velocity, which
			//in turn is determined by the position of the mouse
			double theta = Math.atan2(velocity.y,velocity.z);
			double r = velocity.y*velocity.y + velocity.z*velocity.z;
			//calculate increment matrices
			gamma.y.y = Math.cos(theta);
			gamma.y.z = -Math.sin(theta);
			gamma.z.y = Math.sin(theta);
			gamma.z.z = Math.cos(theta);
			delta.x.x = Math.cos(r);
			delta.x.y = -Math.sin(r);
			delta.y.x = Math.sin(r);
			delta.y.y = Math.cos(r);
			//update orientation matrix
			orientation = orientation.mult(gamma.mult(delta.mult(gamma.transpose())));
			//update distance--controlled by uparrow and downarrow keys
			viewerDistance = Math.max(minViewerDistance,viewerDistance + distanceVelocity);
			baseViewerPosition.x = viewerDistance;
			//render the image
			buffer.clearRect(-250,-250,500,500);
			cubes.render(this,viewerPosition()); //render the image
			screen.drawImage(image,-250,-250,this); //dump buffer to screen
			thread.yield(); //process pending mouse and key events
		}
	}
}
